package gui.panel;

import java.awt.GridBagConstraints;
import java.awt.event.KeyEvent;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.SwingConstants;

import gui.GUI;

/**
 * Handle connection state, allowing to set up and establish a connection as well as displaying it status.
 */
public class ConnectionPanel extends LogPanel {
	private static final long serialVersionUID = 1L; 
	private JLabel addressLabel;
	private JTextField addressText;
	private JLabel portLabel;
	private JTextField portText;
	private JLabel addressPortSepartatorLabel;
	private JToggleButton connectionToggleButton;
	
	private GUI gui;
	
	public void setAddress(String address) {
		if(addressText!=null) addressText.setText(address);
	}
	public void setPort(String port) {
		if(portText!=null) portText.setText(port);
	}
	
	public ConnectionPanel(GUI gui) {
		super();
		this.gui=gui;
		initialize();
	}
	
	@Override
	protected void initialize() {
		super.initialize();
		/*GridBagLayout gbl_main=new GridBagLayout();
		this.setLayout(gbl_main);*/
		
		addressLabel=new JLabel("addressLabel");
		GridBagConstraints gbc_addressLabel=new GridBagConstraints();
		gbc_addressLabel.gridx=0;
		gbc_addressLabel.gridy=0;
		this.add(addressLabel,gbc_addressLabel);
		
		addressText=new JTextField();
		addressText.setHorizontalAlignment(SwingConstants.CENTER);
		addressText.addKeyListener(new java.awt.event.KeyAdapter() {
			@Override
			public void keyPressed(java.awt.event.KeyEvent e) {
				if(e.getKeyCode() == KeyEvent.VK_ENTER){
					if(connectionToggleButton.isSelected()==false) {
						connectionToggleButton.doClick();
					}
				}
			}
		});
		GridBagConstraints gbc_addressText=new GridBagConstraints();
		gbc_addressText.gridx=0;
		gbc_addressText.gridy=1;
		gbc_addressText.weightx=0.5d;
		gbc_addressText.fill=GridBagConstraints.BOTH;
		this.add(addressText,gbc_addressText);
		
		addressPortSepartatorLabel=new JLabel(" : ");
		addressPortSepartatorLabel.setHorizontalAlignment(SwingConstants.CENTER);
		GridBagConstraints gbc_addressPortSepartatorLabel=new GridBagConstraints();
		gbc_addressPortSepartatorLabel.gridx=1;
		gbc_addressPortSepartatorLabel.gridy=1;
		this.add(addressPortSepartatorLabel,gbc_addressPortSepartatorLabel);
		
		portLabel=new JLabel("portLabel");
		GridBagConstraints gbc_portLabel=new GridBagConstraints();
		gbc_portLabel.gridx=2;
		gbc_portLabel.gridy=0;
		this.add(portLabel,gbc_portLabel);
		
		portText=new JTextField();
		portText.setHorizontalAlignment(SwingConstants.CENTER);
		portText.addKeyListener(new java.awt.event.KeyAdapter() {
			@Override
			public void keyPressed(java.awt.event.KeyEvent e) {
				if(e.getKeyCode() == KeyEvent.VK_ENTER){
					if(connectionToggleButton.isSelected()==false) {
						connectionToggleButton.doClick();						
					}
				}
			}
		});
		GridBagConstraints gbc_portText=new GridBagConstraints();
		gbc_portText.gridx=2;
		gbc_portText.gridy=1;
		gbc_portText.weightx=0.5d;
		gbc_portText.fill=GridBagConstraints.BOTH;
		this.add(portText,gbc_portText);
		
		connectionToggleButton=new JToggleButton("connectionToggleButton");
		connectionToggleButton.addActionListener(new java.awt.event.ActionListener() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent e) {
				boolean success=gui.changeConnectionStatus(connectionToggleButton.isSelected(),addressText.getText(),portText.getText());
				if(success==false) {
					connectionToggleButton.setSelected(false);
				}
				setTextConnectionToggleButton();
			}
		});
		GridBagConstraints gbc_connectionToggleButton=new GridBagConstraints();
		gbc_connectionToggleButton.gridx=3;
		gbc_connectionToggleButton.gridy=1;
		gbc_connectionToggleButton.fill=GridBagConstraints.HORIZONTAL;
		this.add(connectionToggleButton,gbc_connectionToggleButton);
		
		setText();
	}
	
	/**
	 * Set the correct text for toggleButtonConnection regarding the selected language and it's status.
	 */
	private void setTextConnectionToggleButton() {
		ResourceBundle resourceBundle=ResourceBundle.getBundle("bundle/Bundle",Locale.getDefault());
		if(connectionToggleButton.isSelected()) {
			connectionToggleButton.setText(resourceBundle.getString("connected"));
		}else {
			connectionToggleButton.setText(resourceBundle.getString("connection"));
		}
	}
	
	@Override
	public void setText() {
		super.setText();
		ResourceBundle resourceBundle=ResourceBundle.getBundle("bundle/Bundle",Locale.getDefault());
		addressLabel.setText(resourceBundle.getString("address"));
		portLabel.setText(resourceBundle.getString("port"));
		setTextConnectionToggleButton();
	}	
}
