package gui.panel;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;

public class LogPanel extends JPanel {
	private static final long serialVersionUID = 1L; 
	private JTextPane logTextPane;
	private StyledDocument logStyleDocument;
	private SimpleAttributeSet normalAttributeSet;
	private SimpleAttributeSet errorAttributeSet;
	private JScrollPane logScrollPane;
	private JButton clearLogButton;
	
	public LogPanel( ) {
		super();
		normalAttributeSet=new SimpleAttributeSet();
		normalAttributeSet.addAttribute(StyleConstants.Foreground,Color.black);
		errorAttributeSet=new SimpleAttributeSet();
		errorAttributeSet.addAttribute(StyleConstants.Foreground,Color.red);
	//	initialize();
	}
	
	/**
	 * Initialize the default state of the different GUI elements.
	 */
	protected void initialize() {
		GridBagLayout gbl_main=new GridBagLayout();
		this.setLayout(gbl_main);
		
		logTextPane=new JTextPane();
		logStyleDocument=logTextPane.getStyledDocument();
		logScrollPane=new JScrollPane(logTextPane);
		GridBagConstraints gbc_logScrollPane=new GridBagConstraints();
		gbc_logScrollPane.gridx=0;
		gbc_logScrollPane.gridy=2;
		gbc_logScrollPane.weightx=1.0d;
		gbc_logScrollPane.weighty=1.0d;
		gbc_logScrollPane.gridwidth=4;
		gbc_logScrollPane.fill=GridBagConstraints.BOTH;
		gbc_logScrollPane.insets=new Insets(5,0,0,0);
		this.add(logScrollPane,gbc_logScrollPane);
		
		clearLogButton=new JButton("clearLogButton");
		clearLogButton.addActionListener(new java.awt.event.ActionListener() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent e) {
				logTextPane.setText("");
			}
		});
		GridBagConstraints gbc_clearLogButton=new GridBagConstraints();
		gbc_clearLogButton.gridx=3;
		gbc_clearLogButton.gridy=3;
		gbc_clearLogButton.fill=GridBagConstraints.HORIZONTAL;
		this.add(clearLogButton,gbc_clearLogButton);
	}
	
	/**
	 * Set the correct text for all graphical elements according to the default {@link Locale}.
	 * @see java.util.Locale#getDefault()
	 */
	public void setText() {
		ResourceBundle resourceBundle=ResourceBundle.getBundle("bundle/Bundle",Locale.getDefault());
		clearLogButton.setText(resourceBundle.getString("clear"));
	}
	
	/**
	 * Append data to the log.
	 * @param data message to append.
	 * @param error indicate if data correspond to an error or not.
	 */
	public void addLog(String data,Boolean error) {
		try {
			if(logStyleDocument.getLength()>0) data="\n"+data;
			if(error) {
				logStyleDocument.insertString(logStyleDocument.getLength(), data, errorAttributeSet);
			}else {
				logStyleDocument.insertString(logStyleDocument.getLength(), data, normalAttributeSet);
			}
		}catch(BadLocationException e) {
			logTextPane.setText(logTextPane.getText()+"\n"+data.trim().trim());
		}
	}
}
