package main;

import java.util.Locale;
import java.util.ResourceBundle;

import generic.Status;
import gui.GUI;
import network.Client;
import network.CommunicationManager;

public class Primary {

	public static void main(String[] args) {
		String serverAddress="127.0.0.1";
		int serverPort=6200;
		String command="";
		
		//Handle user arguments
		if(args.length>0){	
    		for(int i=0;i<args.length;i++){
    			if(args[i].startsWith("ip=")){
    				serverAddress=args[i].substring(3);
    			}
    			if(args[i].startsWith("port=")){
    				serverPort=Integer.valueOf(args[i].substring(5));
    			}
    			if(args[i].startsWith("cmd=")){
    				command=args[i].substring(4);
    			}
    			if(args[i].startsWith("help")||args[i].startsWith("?")){
    				ResourceBundle resourceBundle=ResourceBundle.getBundle("bundle/Bundle",Locale.getDefault());
    				System.out.println(resourceBundle.getString("help_intro")+"\n");
    				System.out.println(resourceBundle.getString("help_argument"));
    				System.out.println("ip=X\t"+resourceBundle.getString("help_ip"));
    				System.out.println("port=X\t"+resourceBundle.getString("help_port"));
    				System.out.println("cmd=X\t"+resourceBundle.getString("help_cmd"));
    				System.out.println("help,?\t"+resourceBundle.getString("help_help"));
    				return;
    			}
    		}
		}
		
		Client client=new Client(serverPort,serverAddress);
		CommunicationManager communicationManager=new CommunicationManager(client);
		
		if(command.isBlank()) {
			GUI gui=new GUI(communicationManager);
			gui.setVisible(true);
			return;
		}
		
		ResourceBundle resourceBundle=ResourceBundle.getBundle("bundle/Bundle",Locale.getDefault());
		
		Status<Object,Exception> connectStatus=communicationManager.connect();
		if(connectStatus.success==false) {
			connectStatus.error.toString();
			return;
		}
		System.out.println(resourceBundle.getString("connectedTo")+": "+serverAddress+":"+serverPort);
		
		Status<Object,Exception> sendStatus=communicationManager.encryptAndSendString(command);
		if(sendStatus.success) {
			Status<String,Exception> receiveStatus=communicationManager.receiveAndDecryptString();
			if(receiveStatus.success) {
				System.out.println(receiveStatus.payload.trim());
			} else {
				receiveStatus.error.toString();
			}
		} else {
			sendStatus.error.toString();
		}
		communicationManager.disconnect();
		System.out.println(resourceBundle.getString("connectionEnded"));
	}

}
