package network;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintStream;
import java.net.Socket;

import generic.Status;
/**
 * TCP client allowing to send and receive Objects.
 */
public class Client {
	private int serverPort;
	private String serverAddress;
	private Socket socket;
	private BufferedReader is;
	private PrintStream os;
	private ObjectInputStream ois;
	private ObjectOutputStream oos;
	
	public Client(){
		serverPort=8033;
		serverAddress="127.0.0.1";
	}
	public Client(int port){
		serverPort=port;
		serverAddress="127.0.0.1";
	}
	public Client(int port,String ip){
		serverPort=port;
		serverAddress=ip;
	}
	
	public int getServerPort() {
		return serverPort;
	}
	public void setServerPort(int serverPort) {
		this.serverPort = serverPort;
	}
	public String getServerAddress() {
		return serverAddress;
	}
	public void setServerAddress(String serverAddress) {
		this.serverAddress = serverAddress;
	}
	public Socket getSocket() {
		return socket;
	}
	public void setSocket(Socket socket) {
		this.socket = socket;
		if(socket!=null) {
			try {
				setIs(new BufferedReader(new InputStreamReader(socket.getInputStream())));
				setOs(new PrintStream(socket.getOutputStream()));
				setOos(new ObjectOutputStream(socket.getOutputStream()));
				setOis(new ObjectInputStream(socket.getInputStream()));
			} catch (Exception e) {
			}
		}
	}
	public BufferedReader getIs() {
		return is;
	}
	public void setIs(BufferedReader is) {
		this.is = is;
	}
	public PrintStream getOs() {
		return os;
	}
	public void setOs(PrintStream os) {
		this.os = os;
	}
	public ObjectInputStream getOis() {
		return ois;
	}
	public void setOis(ObjectInputStream ois) {
		this.ois = ois;
	}
	public ObjectOutputStream getOos() {
		return oos;
	}
	public void setOos(ObjectOutputStream oos) {
		this.oos = oos;
	}
	
	public Status<Object,Exception> connect(){
		Status<Object,Exception> result=new Status<Object,Exception>(true);
		try {
			socket=new Socket(serverAddress,serverPort);
			is=new BufferedReader(new InputStreamReader(socket.getInputStream()));
			os=new PrintStream(socket.getOutputStream());
			ois=new ObjectInputStream(socket.getInputStream());
			oos=new ObjectOutputStream(socket.getOutputStream());
			result.message=socket.getInetAddress().toString().substring(1)+":"+socket.getPort();
		} catch (Exception e) {
			result.success=false;
			result.error=e;
		}
		return result;
	}
	
	public Status<Object,Exception> sendString(String message){
		Status<Object,Exception> result=new Status<Object,Exception>(true);
		try {
			os.println(message);
		} catch (Exception e) {
			result.success=false;
			result.error=e;
		}
		return result;
	}
	
	public Status<String,Exception> receiveString(){
		Status<String,Exception> result=new Status<String,Exception>(true);
		try {
			result.payload=is.readLine();
		} catch (IOException e) {
			result.success=false;
			result.error=e;
		}
		return result;
	}
	
	public Status<Object,IOException> sendObject(Object object){
		Status<Object,IOException> result=new Status<Object,IOException>(true);
		try {
			oos.writeObject(object);
		} catch (IOException e) {
			result.success=false;
			result.error=e;
		}
		return result;
	}
	
	public Status<Object,Exception> receiveObject(){
		Status<Object,Exception> result=new Status<Object,Exception>(true);
		try {
			result.payload=ois.readObject();
		} catch (Exception e) {
			result.success=false;
			result.error=e;
		}
		return result;
	}
	
	public Status<Object,IOException> close(){
		Status<Object,IOException> result=new Status<Object,IOException>(true);
		if(this.socket==null) return result;
		try {
			socket.close();
		} catch (IOException e) {
			result.success=false;
			result.error=e;
		}
		return result;
	}
}