package command;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Serializable;

import generic.Status;

/**
 * Executes specified string command by the environment in which the application is running.
 */
public class ExecuteCommand implements Serializable{
	private static final long serialVersionUID = 1L;
	private Runtime runtime;
	private BufferedReader bufferReader;
	
	public ExecuteCommand(){
		runtime=Runtime.getRuntime();
	}

	public Runtime getRuntime() {
		return runtime;
	}
	public void setRuntime(Runtime runtime) {
		this.runtime = runtime;
	}
	
	/**
	 * Executes the specified string command.
	 * @param command a specified system command.
	 * @return response {@link Status} object.
	 * @see Runtime#exec(String)
	 */
	public Status<String,IOException> executeCommand(String command){
		Status<String,IOException> result=new Status<String, IOException>(false);
		
		String responseLine;
		try {
			Process execution=runtime.exec(command);
			bufferReader=new BufferedReader(new InputStreamReader(execution.getInputStream()));
			while ((responseLine = bufferReader.readLine()) != null){
				if(!(responseLine.isEmpty())){
					if(result.payload==null || result.payload.isBlank()) {
						result.payload=responseLine;
					}else {
						result.payload=result.payload+"\n"+responseLine;
					}
				}
			}
			result.success=true;
		} catch (IOException e) {
			result.success=false;
			result.error=e;
		}
		return result;
	}
}