package gui;

import java.awt.AWTException;
import java.awt.Image;
import java.awt.MenuItem;
import java.awt.PopupMenu;
import java.awt.SystemTray;
import java.awt.Toolkit;
import java.awt.TrayIcon;
import java.awt.event.ActionEvent;
import java.net.InetAddress;
import java.util.Locale;
import java.util.ResourceBundle;

import thread.ServerThread;

public class GUI {
	private TrayIcon trayIcon;
	private Image iconServerStart;
	private Image iconServerStop;
	private PopupMenu trayPopupMenu;
	private MenuItem trayPopupItemQuit;
	private MenuItem trayPopupItemToggleServerState;
	private boolean serverStarted;
	
	private ServerThread serverThread;
	
	public GUI(ServerThread serverThread) {
		serverStarted=false;
		this.serverThread=serverThread;
		initialize();
	}
	
	public void show() {
		try {
			SystemTray.getSystemTray().add(trayIcon);
		} catch (AWTException e) {
			e.printStackTrace();
		}
	}
	
	public void hide() {
		SystemTray.getSystemTray().remove(trayIcon);
	}
	
	private void initialize() {
		if(SystemTray.isSupported()==false) {
			return;
		}
		
		//Setup TrayIcon image
		iconServerStart=Toolkit.getDefaultToolkit().getImage(getClass().getResource("/iconStart.png"));
		iconServerStop=Toolkit.getDefaultToolkit().getImage(getClass().getResource("/iconStop.png"));
		
		//Setup popup menu
		trayPopupItemQuit=new MenuItem("trayPopupItemQuit");
		trayPopupItemQuit.addActionListener(new java.awt.event.ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				serverThread.stopServerSocket();
				serverThread.stopClientsSocket();
				hide();
			}
		});
		trayPopupItemToggleServerState=new MenuItem("trayPopupItemToggleServerState");
		trayPopupItemToggleServerState.addActionListener(new java.awt.event.ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if(serverStarted) {
					serverThread.stopServerSocket();
					serverThread.stopClientsSocket();
				} else {
					if(serverThread.isAlive()==false) {
						serverThread=new ServerThread(serverThread.getServer(), serverThread.getGui());
						serverThread.start();
					}
				}
			}
		});
		trayPopupMenu=new PopupMenu();
		trayPopupMenu.add(trayPopupItemToggleServerState);
		trayPopupMenu.addSeparator();
		trayPopupMenu.add(trayPopupItemQuit);
		
		trayIcon=new TrayIcon(iconServerStop,"CommandPC",trayPopupMenu);
		trayIcon.setImageAutoSize(true);
	
		setText();
	}
	
	/**
	 * Set the correct text for all graphical elements according to the default {@link Locale}.
	 * @see java.util.Locale#getDefault()
	 */
	private void setText() {
		ResourceBundle resourceBundle=ResourceBundle.getBundle("bundle/Bundle",Locale.getDefault());
		trayPopupItemQuit.setLabel(resourceBundle.getString("quit"));
		setTextServerState();
	}
	
	/**
	 * Set the correct text related to server state.
	 */
	private void setTextServerState() {
		ResourceBundle resourceBundle=ResourceBundle.getBundle("bundle/Bundle",Locale.getDefault());
		if(serverStarted) {
			trayPopupItemToggleServerState.setLabel(resourceBundle.getString("serverEndAction"));
		} else {
			trayPopupItemToggleServerState.setLabel(resourceBundle.getString("serverStartAction"));
		}
	}
	
	/**
	 * Indicate to the user that the server has started.
	 */
	public void displayServerStart() {
		serverStarted=true;
		trayIcon.setImage(iconServerStart);
		setTextServerState();
	}
	
	/**
	 * Indicate to the user that the server has stop.
	 */
	public void displayServerStop() {
		serverStarted=false;
		trayIcon.setImage(iconServerStop);
		setTextServerState();
	}

	/**
	 * Indicate to the user that a new client is connected.
	 * @param inetAddress client address
	 * @param port client port
	 * @param localPort local port
	 */
	public void displayStartConnection(InetAddress inetAddress, int port, int localPort) {
		// TODO displayStartConnection()
	}	
}
