package thread;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.Socket;
import java.util.Locale;
import java.util.ResourceBundle;

import command.ExecuteCommand;
import generic.Status;
import network.Client;
import network.CommunicationManager;

/**
 * Thread dedicated to handling a communication with a client.
 */
public class ClientThread extends Thread{
	private CommunicationManager communicationManager;
	private Client client;
	private ExecuteCommand executeCommand;
	private boolean run;
	private boolean osIsWindows;
	
	public boolean isRun() {
		return run;
	}
	public void setRun(boolean run) {
		this.run = run;
	}
	
	public ClientThread(Socket socket) {
		super();
		this.client=new Client();
		this.client.setSocket(socket);
		this.communicationManager=new CommunicationManager(client);
		this.executeCommand=new ExecuteCommand();
		this.run=true;
		this.osIsWindows=System.getProperty("os.name").toLowerCase().startsWith("windows");
	}
	
	private void closeConnection() {
		communicationManager.encryptAndSendString("STOPstopSTOP");
		client.close();
		run=false;
	}

	@Override
	public void run() {
		communicationManager.setupCommunicationKey();

		Status<String,Exception> status;
		while(run) {
			status=client.receiveString();
			if(status.success && status.payload==null && status.error==null) {
				//Client is probably disconnected and client.receiveString() is just "purging" buffer
				closeConnection();
			}
			
			if(run && status.success) {
				status=communicationManager.decryptString(status.payload);
			}
			
			if(run) {
				if(status.success) {
					if(status.payload.compareTo("STOPstopSTOP")==0) {
						closeConnection();
					}else {
						System.out.println(
								ResourceBundle.getBundle("bundle/Bundle",Locale.getDefault()).getString("message")
								+": "+status.payload
						);
						
						Status<String, IOException> resultCommand=executeCommand.executeCommand(status.payload);
						if(resultCommand.success) {
							if(osIsWindows) {
								try {
									resultCommand.payload=new String(resultCommand.payload.getBytes(),"Cp850");
								} catch (UnsupportedEncodingException e) {
									//If encoding fail do nothing as the text will still be mostly legible
								}
							}
							communicationManager.encryptAndSendString(resultCommand.payload);
						}else{
							communicationManager.encryptAndSendString(resultCommand.error.toString());
						}
					}
				}else {
					System.out.println(
							ResourceBundle.getBundle("bundle/Bundle",Locale.getDefault()).getString("messageFail")
							+": "+status.error.toString()
					);
					status.error.printStackTrace();
				}
			}
		}
		System.out.println(ResourceBundle.getBundle("bundle/Bundle",Locale.getDefault()).getString("clientConnectionStop"));
	}
}